import royalnet.engineer as engi
import royalnet_telethon as rt
import pkg_resources
import royalpack.bolts as rb


@rb.capture_errors
@engi.TeleportingConversation
async def version(*, _imp: engi.PDAImplementation, _msg: engi.Message, **__):
    """
    Controlla la versione attuale dei pacchetti di questo bot.
    """

    # noinspection PyListCreation
    msg = [
        f"ℹ️ \uE01BVersioni\uE00B",
    ]

    msg.append("")
    msg.append(f"- \uE01Croyalnet\uE00C \uE01B{pkg_resources.get_distribution('royalnet').version}\uE00B")

    if isinstance(_imp, rt.TelethonPDAImplementation):
        msg.append(f"- \uE01Croyalnet_telethon\uE00C \uE01B{pkg_resources.get_distribution('royalnet_telethon').version}\uE00B")

    msg.append(f"- \uE01Croyalpack\uE00C \uE01B{pkg_resources.get_distribution('royalpack').version}\uE00B")

    await _msg.reply(text="\n".join(msg))


__all__ = ("version",)



"""First revision

Revision ID: c6aacbd5d796
Revises: 
Create Date: 2021-04-25 01:00:49.131019

"""
from alembic import op
import sqlalchemy as sa
import sqlalchemy_utils


# revision identifiers, used by Alembic.
revision = 'c6aacbd5d796'
down_revision = None
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_table('titles',
    sa.Column('uuid', sqlalchemy_utils.types.uuid.UUIDType(), nullable=False),
    sa.Column('name', sa.String(), nullable=False),
    sa.Column('visible', sa.Boolean(), nullable=False),
    sa.Column('locked_description', sa.Text(), nullable=False),
    sa.Column('unlocked_description', sa.Text(), nullable=False),
    sa.PrimaryKeyConstraint('uuid')
    )
    op.create_table('users',
    sa.Column('sub', sa.String(), nullable=False),
    sa.Column('last_update', sqlalchemy_utils.types.arrow.ArrowType(), nullable=False),
    sa.Column('name', sa.String(), nullable=False),
    sa.Column('nickname', sa.String(), nullable=False),
    sa.Column('avatar', sa.String(), nullable=False),
    sa.Column('email', sa.String(), nullable=False),
    sa.Column('bio', sa.Text(), nullable=False),
    sa.Column('color', sqlalchemy_utils.types.color.ColorType(length=20), nullable=False),
    sa.Column('title_fk', sqlalchemy_utils.types.uuid.UUIDType(), nullable=True),
    sa.Column('fiorygi', sa.Integer(), nullable=False),
    sa.ForeignKeyConstraint(['title_fk'], ['titles.uuid'], ),
    sa.PrimaryKeyConstraint('sub')
    )
    op.create_table('accounts_discord',
    sa.Column('user_fk', sa.String(), nullable=False),
    sa.Column('id', sa.BigInteger(), nullable=False),
    sa.Column('username', sa.String(), nullable=False),
    sa.Column('discriminator', sa.SmallInteger(), nullable=False),
    sa.Column('avatar_url', sqlalchemy_utils.types.url.URLType(), nullable=True),
    sa.ForeignKeyConstraint(['user_fk'], ['users.sub'], ),
    sa.PrimaryKeyConstraint('id')
    )
    op.create_table('accounts_league',
    sa.Column('user_fk', sa.String(), nullable=False),
    sa.Column('region', sa.String(), nullable=False),
    sa.Column('puuid', sa.String(), nullable=False),
    sa.Column('summoner_name', sa.String(), nullable=False),
    sa.Column('avatar_id', sa.Integer(), nullable=False),
    sa.ForeignKeyConstraint(['user_fk'], ['users.sub'], ),
    sa.PrimaryKeyConstraint('puuid')
    )
    op.create_table('accounts_osu',
    sa.Column('user_fk', sa.String(), nullable=False),
    sa.Column('id', sa.BigInteger(), nullable=False),
    sa.Column('username', sa.String(), nullable=True),
    sa.Column('avatar_url', sqlalchemy_utils.types.url.URLType(), nullable=True),
    sa.ForeignKeyConstraint(['user_fk'], ['users.sub'], ),
    sa.PrimaryKeyConstraint('id')
    )
    op.create_table('accounts_steam',
    sa.Column('user_fk', sa.String(), nullable=False),
    sa.Column('steamid', sa.BigInteger(), nullable=False),
    sa.Column('persona_name', sa.String(), nullable=False),
    sa.Column('avatar_url', sqlalchemy_utils.types.url.URLType(), nullable=True),
    sa.ForeignKeyConstraint(['user_fk'], ['users.sub'], ),
    sa.PrimaryKeyConstraint('steamid')
    )
    op.create_table('accounts_telegram',
    sa.Column('user_fk', sa.String(), nullable=False),
    sa.Column('id', sa.BigInteger(), nullable=False),
    sa.Column('first_name', sa.String(), nullable=False),
    sa.Column('last_name', sa.String(), nullable=True),
    sa.Column('username', sa.String(), nullable=True),
    sa.Column('avatar_url', sqlalchemy_utils.types.url.URLType(), nullable=True),
    sa.ForeignKeyConstraint(['user_fk'], ['users.sub'], ),
    sa.PrimaryKeyConstraint('id')
    )
    op.create_table('diario_groups',
    sa.Column('id', sa.Integer(), nullable=False),
    sa.Column('saved_by_fk', sa.String(), nullable=False),
    sa.Column('context', sa.Text(), nullable=True),
    sa.ForeignKeyConstraint(['saved_by_fk'], ['users.sub'], ),
    sa.PrimaryKeyConstraint('id')
    )
    op.create_table('fiorygi_transactions',
    sa.Column('id', sa.Integer(), nullable=False),
    sa.Column('minus_fk', sa.String(), nullable=True),
    sa.Column('plus_fk', sa.String(), nullable=True),
    sa.Column('amount', sa.Integer(), nullable=False),
    sa.Column('reason', sa.Text(), nullable=True),
    sa.ForeignKeyConstraint(['minus_fk'], ['users.sub'], ),
    sa.ForeignKeyConstraint(['plus_fk'], ['users.sub'], ),
    sa.PrimaryKeyConstraint('id')
    )
    op.create_table('fiorygi_treasures',
    sa.Column('slug', sa.String(), nullable=False),
    sa.Column('creator_fk', sa.String(), nullable=True),
    sa.Column('finder_fk', sa.String(), nullable=True),
    sa.Column('value', sa.Integer(), nullable=False),
    sa.Column('message', sa.Text(), nullable=True),
    sa.ForeignKeyConstraint(['creator_fk'], ['users.sub'], ),
    sa.ForeignKeyConstraint(['finder_fk'], ['users.sub'], ),
    sa.PrimaryKeyConstraint('slug')
    )
    op.create_table('user_aliases',
    sa.Column('user_fk', sa.String(), nullable=False),
    sa.Column('name', sa.String(), nullable=False),
    sa.ForeignKeyConstraint(['user_fk'], ['users.sub'], ),
    sa.PrimaryKeyConstraint('user_fk', 'name')
    )
    op.create_table('user_title_association',
    sa.Column('user_fk', sa.String(), nullable=True),
    sa.Column('title_fk', sqlalchemy_utils.types.uuid.UUIDType(), nullable=True),
    sa.ForeignKeyConstraint(['title_fk'], ['titles.uuid'], ),
    sa.ForeignKeyConstraint(['user_fk'], ['users.sub'], )
    )
    op.create_table('diario_lines',
    sa.Column('id', sa.Integer(), nullable=False),
    sa.Column('diario_group_fk', sa.Integer(), nullable=False),
    sa.Column('text', sa.Text(), nullable=True),
    sa.Column('media_url', sqlalchemy_utils.types.url.URLType(), nullable=True),
    sa.Column('timestamp', sqlalchemy_utils.types.arrow.ArrowType(), nullable=True),
    sa.Column('spoiler', sa.String(), nullable=True),
    sa.Column('quoted_str', sa.String(), nullable=True),
    sa.Column('quoted_user_fk', sa.String(), nullable=True),
    sa.ForeignKeyConstraint(['diario_group_fk'], ['diario_groups.id'], ),
    sa.ForeignKeyConstraint(['quoted_user_fk'], ['users.sub'], ),
    sa.PrimaryKeyConstraint('id')
    )
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_table('diario_lines')
    op.drop_table('user_title_association')
    op.drop_table('user_aliases')
    op.drop_table('fiorygi_treasures')
    op.drop_table('fiorygi_transactions')
    op.drop_table('diario_groups')
    op.drop_table('accounts_telegram')
    op.drop_table('accounts_steam')
    op.drop_table('accounts_osu')
    op.drop_table('accounts_league')
    op.drop_table('accounts_discord')
    op.drop_table('users')
    op.drop_table('titles')
    # ### end Alembic commands ###



from __future__ import annotations
import typing as t
import sqlalchemy as s
import sqlalchemy.sql as ss
import sqlalchemy.orm as so
import sqlalchemy.ext.declarative as sed
import sqlalchemy_utils as su
import royalnet.alchemist as ra
import colour


Base: sed.declarative_base = sed.declarative_base()


user_title_association = s.Table(
    "user_title_association",
    Base.metadata,
    s.Column("user_fk", s.String, s.ForeignKey("users.sub")),
    s.Column("title_fk", su.UUIDType, s.ForeignKey("titles.uuid"))
)


class User(Base, ra.ColRepr, ra.Updatable):
    """
    An user, as returned by Auth0.
    """
    __tablename__ = "users"

    sub = s.Column(s.String, primary_key=True)
    last_update = s.Column(su.ArrowType, nullable=False)

    name = s.Column(s.String, nullable=False)
    nickname = s.Column(s.String, nullable=False)
    avatar = s.Column(s.String, nullable=False)
    email = s.Column(s.String, nullable=False)

    bio = s.Column(s.Text, nullable=False, default="")
    color = s.Column(su.ColorType, nullable=False, default=colour.Color("#a0ccff"))
    title_fk = s.Column(su.UUIDType, s.ForeignKey("titles.uuid"))
    fiorygi = s.Column(s.Integer, nullable=False, default=0)

    def __str__(self):
        return self.name


class UserAlias(Base, ra.ColRepr, ra.Updatable):
    """
    An alias for an User.
    """
    __tablename__ = "user_aliases"

    user_fk = s.Column(s.String, s.ForeignKey("users.sub"), primary_key=True)
    user = so.relationship("User", backref="aliases")

    name = s.Column(s.String, nullable=False, primary_key=True)

    @so.validates("name")
    def convert_lower(self, key, value: str) -> str:
        return value.lower()

    @classmethod
    def find(cls, session: so.Session, string: str) -> t.Optional[User]:
        ua = session.execute(
            ss.select(cls).where(cls.name == string)
        ).scalar()
        return ua.user if ua else None

    def __str__(self):
        return self.name


class TelegramAccount(Base, ra.ColRepr, ra.Updatable):
    """
    A Telegram account.
    """
    __tablename__ = "accounts_telegram"

    user_fk = s.Column(s.String, s.ForeignKey("users.sub"), nullable=False)
    user = so.relationship("User", backref="telegram_accounts")

    id = s.Column(s.BigInteger, primary_key=True)
    first_name = s.Column(s.String, nullable=False)
    last_name = s.Column(s.String)
    username = s.Column(s.String)
    avatar_url = s.Column(su.URLType)

    def name(self) -> str:
        if self.username is not None:
            return f"{self.username}"
        elif self.last_name is not None:
            return f"{self.first_name} {self.last_name}"
        else:
            return f"{self.first_name}"

    def mention(self) -> str:
        if self.username is not None:
            return f"@{self.username}"
        else:
            return f"[{self.name}](tg://user?id={self.tg_id})"

    def __str__(self):
        return self.name()


class DiscordAccount(Base, ra.ColRepr, ra.Updatable):
    """
    A Discord account.
    """
    __tablename__ = "accounts_discord"

    user_fk = s.Column(s.String, s.ForeignKey("users.sub"), nullable=False)
    user = so.relationship("User", backref="discord_accounts")

    id = s.Column(s.BigInteger, primary_key=True)
    username = s.Column(s.String, nullable=False)
    discriminator = s.Column(s.SmallInteger, nullable=False)
    avatar_url = s.Column(su.URLType)

    def name(self) -> str:
        return f"{self.username}#{self.discriminator}"

    def __str__(self):
        return self.name()


class SteamAccount(Base, ra.ColRepr, ra.Updatable):
    """
    A Steam account.
    """
    __tablename__ = "accounts_steam"

    user_fk = s.Column(s.String, s.ForeignKey("users.sub"), nullable=False)
    user = so.relationship("User", backref="steam_accounts")

    steamid = s.Column(s.BigInteger, primary_key=True)
    persona_name = s.Column(s.String, nullable=False)
    avatar_url = s.Column(su.URLType)

    # TODO: make steamid return steam.steamid.SteamID objects

    def __str__(self):
        return self.persona_name


class OsuAccount(Base, ra.ColRepr, ra.Updatable):
    """
    An osu! account.
    """
    __tablename__ = "accounts_osu"

    user_fk = s.Column(s.String, s.ForeignKey("users.sub"), nullable=False)
    user = so.relationship("User", backref="osu_accounts")

    id = s.Column(s.BigInteger, primary_key=True)
    username = s.Column(s.String)
    avatar_url = s.Column(su.URLType)

    def __str__(self):
        return self.username


class LeagueAccount(Base, ra.ColRepr, ra.Updatable):
    """
    A League of Legends account.
    """
    __tablename__ = "accounts_league"

    user_fk = s.Column(s.String, s.ForeignKey("users.sub"), nullable=False)
    user = so.relationship("User", backref="league_accounts")

    region = s.Column(s.String, nullable=False)
    puuid = s.Column(s.String, primary_key=True)
    summoner_name = s.Column(s.String, nullable=False)
    avatar_id = s.Column(s.Integer, nullable=False)

    def __str__(self):
        return self.summoner_name


class Title(Base, ra.ColRepr, ra.Updatable):
    """
    Title available for users to unlock.
    """
    __tablename__ = "titles"

    uuid = s.Column(su.UUIDType, primary_key=True)

    name = s.Column(s.String, nullable=False)
    visible = s.Column(s.Boolean, nullable=False)
    locked_description = s.Column(s.Text, nullable=False)
    unlocked_description = s.Column(s.Text, nullable=False)

    unlocked_by = so.relationship("User", secondary=user_title_association, backref="unlocked_titles")

    def __str__(self):
        return self.name


class DiarioGroup(Base, ra.ColRepr, ra.Updatable):
    """
    Group of Diario entries.
    """
    __tablename__ = "diario_groups"

    id = s.Column(s.Integer, primary_key=True)

    saved_by_fk = s.Column(s.String, s.ForeignKey("users.sub"), nullable=False)
    saved_by = so.relationship("User", backref="diario_groups_saved")

    context = s.Column(s.Text)


class DiarioLine(Base, ra.ColRepr, ra.Updatable):
    """
    Single Diario quote.
    """
    __tablename__ = "diario_lines"

    id = s.Column(s.Integer, primary_key=True)

    diario_group_fk = s.Column(s.Integer, s.ForeignKey("diario_groups.id"), nullable=False)
    diario_group = so.relationship("DiarioGroup", backref="lines")

    text = s.Column(s.Text)
    media_url = s.Column(su.URLType)
    timestamp = s.Column(su.ArrowType)
    spoiler = s.Column(s.String)
    quoted_str = s.Column(s.String)
    quoted_user_fk = s.Column(s.String, s.ForeignKey("users.sub"))
    quoted_user = so.relationship("DiarioGroup", backref="diario_quoted_in")


class Transaction(Base, ra.ColRepr, ra.Updatable):
    """
    Single fiorygi transaction.
    """
    __tablename__ = "fiorygi_transactions"

    id = s.Column(s.Integer, primary_key=True)

    minus_fk = s.Column(s.String, s.ForeignKey("users.sub"))
    minus = so.relationship("User", foreign_keys=(minus_fk,), backref="transactions_minus")

    plus_fk = s.Column(s.String, s.ForeignKey("users.sub"))
    plus = so.relationship("User", foreign_keys=(plus_fk,), backref="transactions_plus")

    amount = s.Column(s.Integer, nullable=False)
    reason = s.Column(s.Text)
    timestamp = s.Column(su.ArrowType)


class Treasure(Base, ra.ColRepr, ra.Updatable):
    """
    A treasure code which rewards fiorygi.
    """
    __tablename__ = "fiorygi_treasures"

    slug = s.Column(s.String, primary_key=True)

    creator_fk = s.Column(s.String, s.ForeignKey("users.sub"))
    creator = so.relationship("User", foreign_keys=(creator_fk,))
    creation_time = s.Column(su.ArrowType, nullable=False)

    finder_fk = s.Column(s.String, s.ForeignKey("users.sub"))
    finder = so.relationship("User", foreign_keys=(finder_fk,))
    find_time = s.Column(su.ArrowType)

    value = s.Column(s.Integer, nullable=False)
    message = s.Column(s.Text)



import io

import aiohttp
import royalnet.engineer as engi
import royalpack.bolts as rb


@rb.capture_errors
@engi.TeleportingConversation
async def dog_any(*, _msg: engi.Message, **__):
    """
    Invia un doggo in chat! 🐶
    """
    async with aiohttp.ClientSession() as session:
        async with session.get("https://dog.ceo/api/breeds/image/random") as response:
            result = await response.json()
            url = result["message"]
        async with session.get(url) as response:
            img = await response.content.read()
    await _msg.reply(files=[io.BytesIO(img)])


_breeds = [
    "affenpinscher",
    "african",
    "airedale",
    "akita",
    "appenzeller",
    "australian/shepherd",
    "basenji",
    "beagle",
    "bluetick",
    "borzoi",
    "bouvier",
    "boxer",
    "brabancon",
    "briard",
    "buhund/norwegian",
    "bulldog/boston",
    "bulldog/english",
    "bulldog/french",
    "bullterrier/staffordshire",
    "cairn",
    "cattledog/australian",
    "chihuahua",
    "chow",
    "clumber",
    "cockapoo",
    "collie/border",
    "coonhound",
    "corgi/cardigan",
    "cotondetulear",
    "dachshund",
    "dalmatian",
    "dane/great",
    "deerhound/scottish",
    "dhole",
    "dingo",
    "doberman",
    "elkhound/norwegian",
    "entlebucher",
    "eskimo",
    "finnish/lapphund",
    "frise/bichon",
    "germanshepherd",
    "greyhound/italian",
    "groenendael",
    "havanese",
    "hound/afghan",
    "hound/basset",
    "hound/blood",
    "hound/english",
    "hound/ibizan",
    "hound/plott",
    "hound/walker",
    "husky",
    "keeshond",
    "kelpie",
    "komondor",
    "kuvasz",
    "labrador",
    "leonberg",
    "lhasa",
    "malamute",
    "malinois",
    "maltese",
    "mastiff/bull",
    "mastiff/english",
    "mastiff/tibetan",
    "mexicanhairless",
    "mix",
    "mountain/bernese",
    "mountain/swiss",
    "newfoundland",
    "otterhound",
    "ovcharka/caucasian",
    "papillon",
    "pekinese",
    "pembroke",
    "pinscher/miniature",
    "pitbull",
    "pointer/german",
    "pointer/germanlonghair",
    "pomeranian",
    "poodle/miniature",
    "poodle/standard",
    "poodle/toy",
    "pug",
    "puggle",
    "pyrenees",
    "redbone",
    "retriever/chesapeake",
    "retriever/curly",
    "retriever/flatcoated",
    "retriever/golden",
    "ridgeback/rhodesian",
    "rottweiler",
    "saluki",
    "samoyed",
    "schipperke",
    "schnauzer/giant",
    "schnauzer/miniature",
    "setter/english",
    "setter/gordon",
    "setter/irish",
    "sheepdog/english",
    "sheepdog/shetland",
    "shiba",
    "shihtzu",
    "spaniel/blenheim",
    "spaniel/brittany",
    "spaniel/cocker",
    "spaniel/irish",
    "spaniel/japanese",
    "spaniel/sussex",
    "spaniel/welsh",
    "springer/english",
    "stbernard",
    "terrier/american",
    "terrier/australian",
    "terrier/bedlington",
    "terrier/border",
    "terrier/dandie",
    "terrier/fox",
    "terrier/irish",
    "terrier/kerryblue",
    "terrier/lakeland",
    "terrier/norfolk",
    "terrier/norwich",
    "terrier/patterdale",
    "terrier/russell",
    "terrier/scottish",
    "terrier/sealyham",
    "terrier/silky",
    "terrier/tibetan",
    "terrier/toy",
    "terrier/westhighland",
    "terrier/wheaten",
    "terrier/yorkshire",
    "vizsla",
    "waterdog/spanish",
    "weimaraner",
    "whippet",
    "wolfhound/irish",
]


@engi.TeleportingConversation
async def dog_breed(*, _msg: engi.Message, breed: str, **__):
    """
    Invia un doggo di una razza specifica in chat! 🐶
    """
    breed = breed.lower()
    if breed not in _breeds:
        await _msg.reply(text="⚠️ La razza che hai specificato non esiste nel database.")

    async with aiohttp.ClientSession() as session:
        async with session.get(f"https://dog.ceo/api/breed/{breed}/images/random") as response:
            result = await response.json()
            url = result["message"]
        async with session.get(url) as response:
            img = await response.content.read()
    await _msg.reply(files=[io.BytesIO(img)])


@engi.TeleportingConversation
async def dog_breedlist(*, _msg: engi.Message, **__):
    """
    Elenca tutte le razze di dogghi disponibili! 🐶
    """
    msg = [
        "🐶 \uE01BRazze disponibili:\uE00B",
        ", ".join(_breeds),
    ]

    await _msg.reply(text="\n".join(msg))


__all__ = ("dog_any", "dog_breed", "dog_breedlist")



"""Add timestamp to transactions

Revision ID: e6b0d97063a1
Revises: c6aacbd5d796
Create Date: 2021-04-25 02:18:44.248837

"""
from alembic import op
import sqlalchemy as sa
import sqlalchemy_utils


# revision identifiers, used by Alembic.
revision = 'e6b0d97063a1'
down_revision = 'c6aacbd5d796'
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column('fiorygi_transactions', sa.Column('timestamp', sqlalchemy_utils.types.arrow.ArrowType(), nullable=True))
    op.add_column('fiorygi_treasures', sa.Column('creation_time', sqlalchemy_utils.types.arrow.ArrowType(), nullable=False))
    op.add_column('fiorygi_treasures', sa.Column('find_time', sqlalchemy_utils.types.arrow.ArrowType(), nullable=False))
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column('fiorygi_treasures', 'find_time')
    op.drop_column('fiorygi_treasures', 'creation_time')
    op.drop_column('fiorygi_transactions', 'timestamp')
    # ### end Alembic commands ###



import royalnet.engineer as engi
import royalpack.bolts as rb
import datetime
import random


ANSWERS = [
    # Cerchiamo di tenere bilanciate le tre colonne, o almeno le prime due.
    # Se avete un'idea ma metterebbe troppe opzioni in un'unica categoria, mettetela sotto commento.

    # risposte "sì": 19
    "🔵 Sì.",
    "🔵 Decisamente sì!",
    "🔵 Uhm, secondo me sì.",
    "🔵 Sì! Sì! SÌ!",
    "🔵 Yup.",
    "🔵 Direi proprio di sì.",
    "🔵 Assolutamente sì.",
    "🔵 Ma certo!",
    "🔵 Esatto!",
    "🔵 Senz'altro!",
    "🔵 Ovviamente.",
    "🔵 Questa domanda ha risposta affermativa.",
    "🔵 Hell yeah.",
    "🔵 YES! YES! YES!",
    "🔵 yusssssss",
    "🔵 Non vedo perchè no",
    "🔵 Ha senso, ha perfettamente senso, nulla da obiettare, ha senso.",
    "🔵 Yos!",
    "🔵 Sì, ma tienilo segreto...",

    # risposte "no": 19
    "❌ No.",
    "❌ Decisamente no!",
    "❌ Uhm, secondo me sì. No, aspetta, ci ho ripensato. È un no.",
    "❌ No, no, e ancora NO!",
    "❌ Nope.",
    "❌ Direi proprio di no.",
    "❌ Assolutamente no.",
    "❌ Certo che no!",
    "❌ Neanche per idea!",
    "❌ Neanche per sogno!",
    "❌ Niente affatto!",
    "❌ Questa domanda ha risposta negativa.",
    "❌ Hell no.",
    "❌ NO! NO! NO!",
    "❌ lolno",
    "❌ NEIN NEIN NEIN NEIN",
    "❌ Delet dis",
    "❌ Nopety nope!",
    "❌ No, ma tienilo segreto.",

    # risposte "boh": 19
    "❔ Boh.",
    "❔ E io che ne so?!",
    "❔ Non so proprio rispondere.",
    "❔ Non lo so...",
    "❔ Mi avvalgo della facoltà di non rispondere.",
    "❔ Non parlerò senza il mio avvocato!",
    "❔ Dunno.",
    "❔ Perché lo chiedi a me?",
    "❔ Ah, non lo so io!",
    r"❔ ¯\_(ツ)_/¯",
    "❔ No idea.",
    "❔ Dunno.",
    "❔ Boooooh!",
    "❔ Non ne ho la più pallida idea.",
    "❔ No comment.",
    "❔ maibi",
    "❔ maibi not",
    "❔ idk dude",
    "❔ Non mi è permesso condividere questa informazione.",
]


@rb.capture_errors
@engi.TeleportingConversation
async def answer(*, _msg: engi.Message, **__):
    """
    Fai una domanda al bot, che possa essere risposta con un sì o un no: lui ti risponderà!
    """

    h = hash(datetime.datetime.now())
    r = random.Random(x=h)

    message = r.sample(ANSWERS, 1)[0]

    await _msg.reply(text=message)


# Objects exported by this module
__all__ = (
    "answer",
)



import logging
import re
import royalpack.bolts as rb

import royalnet.engineer as engi

log = logging.getLogger(__name__)


@rb.capture_errors
@engi.TeleportingConversation
async def ship(*, _msg: engi.Message, first: str, second: str, **__):
    """
    Shippa insieme due persone! 💞
    """
    log.info(f"Shipping: {first!r} + {second!r}")

    # Convert the names to lowercase
    first = first.lower()
    second = second.lower()

    log.debug(f"Lowercased: {first!r} + {second!r}")

    # Decide the number of groups to keep
    first_groups = len(first) // 5
    second_groups = len(second) // 5

    log.debug(f"Keeping first:{first} second:{second} groups")

    # Try to get a match
    first_match = re.search(rf"^(?:[^aeiou]*[aeiou]){{,{first_groups}}}", first)
    second_match = re.search(rf"(?:[^aeiou\s]*[aeiou]){{,{second_groups}}}[a-z]?$", second)

    log.debug(f"Matches: {first_match!r} + {second_match!r}")

    # Get the matched characters if the matches were successful, or cut the names in half if they weren't
    first_crop = first_match.group(0) if first_match else first[:(len(first) // 2)]
    second_crop = second_match.group(0) if second_match else second[(len(second) // 2):]

    log.debug(f"Cropped: {first_crop!r} + {second_crop!r}")

    # Combine the two parts
    combined = f"{first_crop}{second_crop}"

    log.info(f"Combined: {combined!r}")

    # Send the message to the chat
    log.debug(f"Sending ship to the chat...")
    await _msg.reply(
        text=f"💞 {first.capitalize()} + {second.capitalize()} = \uE01B{combined.capitalize()}\uE00B"
    )


# Objects exported by this module
__all__ = (
    "ship",
)



import royalnet.engineer as engi
import royalpack.bolts as rb


@rb.capture_errors
@engi.TeleportingConversation
async def ahnonlosoio(*, _msg: engi.Message, **__):
    """
    Ah, non lo so io!
    """
    await _msg.reply(text=r"¯\_(ツ)_/¯ Ah, non lo so io!")


__all__ = ("ahnonlosoio",)



"""

"""

from __future__ import annotations

import functools
import logging

import royalnet.engineer as engi
import sqlalchemy.orm as so

import royalpack.database as db

log = logging.getLogger(__name__)


def with_target():
    """
    .. todo:: Document this.
    """

    def decorator(f):
        @functools.wraps(f)
        async def decorated(_msg: engi.Message, _session: so.Session, target: str, **f_kwargs):
            user = db.UserAlias.find(session=_session, string=target)
            if user is None:
                await _msg.reply(text=f"⚠️ L'utente specificato non esiste.")
                return

            return await f(_msg=_msg, _session=_session, **f_kwargs, _target=user)

        return decorated

    return decorator


__all__ = (
    "with_target",
)



import royalnet.engineer as engi
import royalpack.bolts as rb


@rb.capture_errors
@engi.TeleportingConversation
async def ping(*, _msg: engi.Message, **__):
    """
    Gioca a ping pong con il bot. 🏓
    """
    await _msg.reply(text="🏓 Pong!")


__all__ = ("ping",)



import royalnet.engineer as engi
import royalpack.database as db
import royalpack.bolts as rb


@rb.capture_errors
@engi.use_database(db.lazy_session_class)
@rb.use_ryglogin(allow_anonymous=True)
@engi.TeleportingConversation
async def whoami(*, _msg: engi.Message, _user: db.User, **__):
    """
    Scopri con che RYGaccount sei loggato.
    """

    if _user:
        await _msg.reply(text=f"☀️ Sei loggato come \uE01B{_user.name}\uE00B!")
    else:
        await _msg.reply(text="☁️ Non hai effettuato il login.")


__all__ = ("whoami",)



import discord
import royalnet.royaltyping as t
import royalnet.engineer as engi
import sqlalchemy.orm as so
import royalpack.database as db
import royalpack.config as cfg
import royalnet_telethon.bullet.contents
import royalnet_discordpy
import royalpack.bolts as rb
import aiohttp
import asyncio
import logging
import arrow
import async_timeout

log = logging.getLogger(__name__)


@rb.capture_errors
@engi.use_database(db.lazy_session_class)
@engi.TeleportingConversation
async def login(*, _msg: engi.Message, _session: so.Session, _imp, **__):
    """
    Fai il login al tuo RYGaccount.
    """

    log.debug("Evaluating config...")
    config = cfg.lazy_config.evaluate()

    private = await enforce_private_message(msg=_msg)

    async with aiohttp.ClientSession() as http_session:

        dc = await device_code_request(
            http_session=http_session,
            client_id=config["auth.client.id"],
            device_url=config["auth.url.device"],
            scopes=["profile", "email", "openid"],
        )

        await prompt_login(
            channel=private,
            verification_url=dc['verification_uri_complete'],
            user_code=dc['user_code']
        )

        try:
            async with async_timeout.timeout(dc["expires_in"]):
                at = await device_code_exchange(
                    http_session=http_session,
                    client_id=config["auth.client.id"],
                    token_url=config["auth.url.token"],
                    device_code=dc["device_code"],
                    sleep_time=9
                )
        except asyncio.TimeoutError:
            await notify_expiration(
                channel=private
            )
            return

        ui = await get_user_info(
            http_session=http_session,
            userinfo_url=config["auth.url.userinfo"],
            token_type=at["token_type"],
            access_token=at["access_token"],
        )

    user = await register_user_generic(session=_session, user_info=ui)
    uas = await register_user_alias(session=_session, user_info=ui)

    log.debug(f"Committing session...")
    _session.commit()

    log.debug(f"Done, notifying the user...")
    await private.send_message(text=f"✅ Login riuscito! Sei loggato come {user.name}!")

    if isinstance(_imp, royalnet_telethon.TelethonPDAImplementation):
        sender = await _msg.sender
        tg = await register_user_telethon(session=_session, user_info=ui, telethon_user=sender._user)

        log.debug(f"Committing session...")
        _session.commit()

        log.debug(f"Done, notifying the user...")
        await private.send_message(text=f"↔️ Sincronizzazione con Telegram riuscita! Sei loggato come {tg.mention()}!")

    elif isinstance(_imp, royalnet_discordpy.DiscordpyPDAImplementation):
        sender = await _msg.sender
        ds = await register_user_discord(session=_session, user_info=ui, discord_user=sender._user)

        log.debug(f"Committing session...")
        _session.commit()

        log.debug(f"Done, notifying the user...")
        await private.send_message(text=f"↔️ Sincronizzazione con Discord riuscita! Sei loggato come <@{ds.id}>!")


async def enforce_private_message(msg: engi.Message) -> engi.Channel:
    """
    Get the private chat for an user and notify them of the switch.

    :param msg: The :class:`~.engi.Message` to reply to.
    :return: The private :class:`~.engi.Channel`.
    """

    log.debug("Sliding into DMs...")

    sender: engi.User = await msg.sender
    current: engi.Channel = await msg.channel
    private: engi.Channel = await sender.slide()
    if hash(current) != hash(private):
        await msg.reply(text="👤 Ti sto inviando un messaggio in chat privata contenente le istruzioni per il login!")
    return private


async def device_code_request(
        http_session: aiohttp.ClientSession,
        client_id: str,
        device_url: str,
        scopes: list[str],
) -> t.JSON:
    """
    Request a OAuth2 device code (which can be exchanged for an access token once the user has given us the
    authorization to do so).

    :param http_session: The :class:`aiohttp.ClientSession` to use.
    :param client_id: The OAuth2 Client ID.
    :param device_url: The URL where device codes can be obtained.
    :param scopes: A :class:`list` of scopes to require from the user.

    :return: The JSON response received from the Identity Provider.
    """

    log.debug("Requesting device code...")

    async with http_session.post(device_url, data={
        "client_id": client_id,
        "scope": " ".join(scopes),
    }) as request:
        return await request.json()


async def prompt_login(channel: engi.Channel, verification_url: str, user_code: str) -> None:
    """
    Ask the user to login.

    :param channel: The :class:`~.engi.Channel` to send the message in.
    :param verification_url: The URL where the user can approve / reject the token.
    :param user_code: Human-friendly view of the device code.
    """

    log.debug("Asking user to login...")

    await channel.send_message(
        text=f"🌍 Effettua il RYGlogin al seguente URL, poi premi Confirm:\n"
             f"{verification_url}\n"
             f"\n"
             f"\uE011(Codice: {user_code})\uE001"
    )


async def device_code_exchange(
        http_session: aiohttp.ClientSession,
        client_id: str,
        token_url: str,
        device_code: str,
        sleep_time: float,
):
    """
    Check if the user has authorized the device code, and try to exchange it for an access token.

    :return: The JSON response received from the Identity Provider.
    """

    log.debug("Starting validation process...")

    while True:
        log.debug(f"Sleeping for {sleep_time}s...")
        await asyncio.sleep(sleep_time)

        async with http_session.post(token_url, data={
            "client_id": client_id,
            "grant_type": "urn:ietf:params:oauth:grant-type:device_code",
            "device_code": device_code,
        }) as request:
            response = await request.json()
            if "error" in response:
                log.debug(f"Response returned error {response['error']!r}, retrying...")
                continue
            elif "access_token" in response:
                log.debug(f"Obtained access token!")
                return response
            else:
                log.error(f"Didn't get an access token, but didn't get an error either?!")
                continue


async def get_user_info(
        http_session: aiohttp.ClientSession,
        userinfo_url: str,
        token_type: str,
        access_token: str,
):
    """
    Get the userinfo of an user.

    :param http_session: The :class:`aiohttp.ClientSession` to use.
    :param userinfo_url: The URL where the user info is obtained.
    :param token_type: The type of the token returned by the Identity Provider (usually ``"Bearer"``)
    :param access_token: The access token to use.
    :return:
    """

    log.debug("Getting user info...")

    async with http_session.post(userinfo_url, headers={
        "Authorization": f"{token_type} {access_token}"
    }) as request:
        return await request.json()


async def notify_expiration(channel: engi.Channel) -> None:
    """
    Notify the user of the device code expiration.

    :param channel: The :class:`~.engi.Channel` to send the message in.
    """

    log.debug("Notifying the user of the expiration...")

    await channel.send_message(
        text=f"🕒 Il codice dispositivo è scaduto e il login è stato annullato. "
             f"Fai il login più in fretta la prossima volta! :)",
    )


async def register_user_generic(
        session: so.Session,
        user_info: dict[str, t.Any],
) -> db.User:
    """
    Sync the user info with the data inside the database.

    :param session: The :class:`~.so.Session` to use.
    :param user_info: The user_info obtained by the Identity Provider.
    :return: The created/updated :class:`.db.User`.
    """

    log.debug("Syncing generic user...")

    user = db.User(
        sub=user_info['sub'],
        last_update=arrow.now(),
        name=user_info['name'],
        nickname=user_info['nickname'],
        avatar=user_info['picture'],
        email=user_info['email'],
    )
    session.merge(user)

    return user


async def register_user_alias(
        session: so.Session,
        user_info: dict[str, t.Any],
):
    """
    .. todo:: Document this.
    """

    log.debug("Syncing user aliases...")

    uas = [
        db.UserAlias(
            user_fk=user_info["sub"],
            name=user_info["name"]
        ),
        db.UserAlias(
            user_fk=user_info["sub"],
            name=user_info["nickname"]
        ),
        db.UserAlias(
            user_fk=user_info["sub"],
            name=user_info["email"]
        ),
        db.UserAlias(
            user_fk=user_info["sub"],
            name=user_info["sub"]
        ),
    ]

    for ua in uas:
        session.merge(ua)

    return uas


async def register_user_telethon(
        session: so.Session,
        user_info: dict[str, t.Any],
        telethon_user,
) -> db.TelegramAccount:
    """
    Sync an user's Telegram account via a Telethon message.

    :param session: The :class:`~.so.Session` to use.
    :param user_info: The user_info obtained by the Identity Provider.
    :param telethon_user: The telethon user to base the user data on.
    :return: The created/updated :class:`~.db.TelegramAccount`
    """

    log.debug("Syncing telethon user...")

    tg = db.TelegramAccount(
        user_fk=user_info["sub"],
        id=telethon_user.id,
        first_name=telethon_user.first_name,
        last_name=telethon_user.last_name,
        username=telethon_user.username,
        avatar_url=None,  # TODO: avatars
    )
    session.merge(tg)
    return tg


async def register_user_discord(
        session: so.Session,
        user_info: dict[str, t.Any],
        discord_user: discord.User,
) -> db.DiscordAccount:
    """
    Sync an user's Discord account via a Discord.py message.

    :param session: The :class:`~.so.Session` to use.
    :param user_info: The user_info obtained by the Identity Provider.
    :param discord_user: The Discord.py user to base the user data on.
    :return: The created/updated :class:`~.db.DiscordAccount`
    """

    log.debug("Syncing telethon user...")

    ds = db.DiscordAccount(
        user_fk=user_info["sub"],
        id=discord_user.id,
        username=discord_user.name,
        discriminator=discord_user.discriminator,
        avatar_url=str(discord_user.avatar_url),
    )
    session.merge(ds)
    return ds


__all__ = ("login",)



import royalnet.engineer as engi
import royalpack.bolts as rb


@rb.capture_errors
@engi.TeleportingConversation
async def color(*, _msg: engi.Message, **__):
    """
    Invia un colore in chat...?
    """

    await _msg.reply(
        text="\uE011I am sorry, unknown error occured during working with your request, Admin were notified\uE001"
    )


__all__ = ("color",)



import io
import logging

import aiohttp
import royalnet.engineer as engi
import royalpack.bolts as rb

log = logging.getLogger(__name__)


@rb.capture_errors
@engi.TeleportingConversation
async def cat(*, _msg: engi.Message, **__):
    """
    Invia un gatto in chat! 🐈
    """

    log.debug("Creating a new HTTP session")
    async with aiohttp.ClientSession() as session:

        log.info("Making a GET request to The Cat API Image Search")
        async with session.get("https://api.thecatapi.com/v1/images/search") as response:

            log.debug("Ensuring the request was successful")
            if response.status >= 400:
                log.error(f"The Cat API returned an HTTP error: {response.status}")
                await _msg.reply(
                    text="⚠️ Couldn't request a cat from https://thecatapi.com :("
                )
                return

            log.debug("Reading the JSON received from The Cat API")
            try:
                result = await response.json()
            except aiohttp.ContentTypeError:
                log.error(f"Couldn't decode received JSON from The Cat API")
                await _msg.reply(
                    text="⚠️ Couldn't understand what the cat from https://thecatapi.com was saying :("
                )
                return

        # Example result:
        # [
        #     {
        #         "breeds": [],
        #         "id": "MjAzMjY3MQ",
        #         "url": "https://cdn2.thecatapi.com/images/MjAzMjY3MQ.jpg",
        #         "width": 557,
        #         "height": 724
        #     }
        # ]

        log.debug("Ensuring at least one image was received")
        if len(result) == 0:
            log.error("Didn't receive any image from The Cat API")
            await _msg.reply(
                text="⚠️ I couldn't find any cats at https://thecatapi.com :("
            )
            return

        # Select the first image received
        selected_cat = result[0]
        log.debug(f"Selected {selected_cat!r}")

        log.debug("Ensuring an image url is available")
        if "url" not in selected_cat:
            log.error("Image received from The Cat API did not have any URL")
            await _msg.reply(
                text="⚠️ I found a cat at https://thecatapi.com, but I couldn't find its image :("
            )
            return

        # Download the cat image
        log.info("Making a GET request to retrieve a The Cat API image")
        async with session.get(selected_cat["url"]) as response:

            log.debug("Reading image bytes into memory")
            img = io.BytesIO()
            while img_data := await response.content.read(8192):
                img.write(img_data)
            img.seek(0)

    log.debug("Sending image in the chat")
    await _msg.reply(files=[img])


# Objects exported by this module
__all__ = (
    "cat",
)



import royalnet.engineer as engi
import royalpack.bolts as rb


@rb.capture_errors
@engi.TeleportingConversation
async def man(*, _msg: engi.Message, _router: engi.Router, commandname: str, **__):
    """
    Visualizza aiuto per un comando.

    Non funziona ancora correttamente per i multicomandi, come /dog: https://github.com/Steffo99/royalnet/issues/11 !
    """

    # TODO: Change this when royalnet/#11 is fixed!

    if not (command := _router.by_name.get(commandname)):
        await _msg.reply(text="⚠️ Il comando che hai specificato non esiste.")
        return

    try:
        command = command.__getattribute__("bare_function")
    except AttributeError:
        pass

    msg = [
        f"ℹ️ Manuale di \uE011{commandname}\uE001:",
        f"{command.__doc__}",
    ]

    await _msg.reply(text="\n".join(msg))


__all__ = ("man",)



import royalnet.royaltyping as t
import royalnet.engineer as engi
import royalpack.database as db
import royalpack.bolts as rb
import sqlalchemy.sql as ss
import functools
import arrow


@rb.capture_errors
@engi.use_database(db.lazy_session_class)
@rb.use_ryglogin(allow_anonymous=False)
@engi.TeleportingConversation
async def fiorygi_balance_self(*, _user: db.User, _msg: engi.Message, **__):
    """
    Visualizza il tuo portafoglio attuale di fiorygi.
    """

    await _msg.reply(text=f"💰 Attualmente, possiedi \uE01Bƒ {_user.fiorygi}\uE00B.")


@rb.capture_errors
@engi.use_database(db.lazy_session_class)
@rb.with_target()
@engi.TeleportingConversation
async def fiorygi_balance_other(*, _target: db.User, _session: db.SessionType, _msg: engi.Message, **__):
    """
    Visualizza il portafoglio di fiorygi di un altro membro.
    """

    await _msg.reply(text=f"💰 {_target} possiede \uE01Bƒ {_target.fiorygi}\uE00B.")


def render(transaction: db.Transaction, user: db.User):
    row = []

    if transaction.plus == user:
        is_plus = True
        other = transaction.minus
    else:
        is_plus = False
        other = transaction.plus

    if transaction.amount != 0:
        row.append(f"{'➕' if is_plus else '➖'} \uE01Bƒ {transaction.amount}\uE00B")

    if other is not None:
        row.append(f"\uE011{'da' if is_plus else 'a'} {other}\uE001")

    if transaction.reason:
        row.append(f"{transaction.reason}")

    return " - ".join(row)


@rb.capture_errors
@engi.use_database(db.lazy_session_class)
@rb.use_ryglogin(allow_anonymous=False)
@engi.TeleportingConversation
async def fiorygi_transactions_self(*, _session: db.SessionType, _user: db.User, _msg: engi.Message, **__):
    """
    Visualizza le ultime 10 transazioni del tuo portafoglio.
    """

    transactions = _session.execute(
        ss.select(
            db.Transaction
        ).where(
            ss.or_(
                db.Transaction.minus == _user,
                db.Transaction.plus == _user,
            )
        ).order_by(
            db.Transaction.timestamp.desc()
        ).limit(
            10
        )
    ).scalars()

    msg = map(functools.partial(render, user=_user), transactions)

    await _msg.reply(text="\n\n".join(msg))


@rb.capture_errors
@engi.use_database(db.lazy_session_class)
@rb.with_target()
@engi.TeleportingConversation
async def fiorygi_transactions_other(*, _session: db.SessionType, _target: db.User, _msg: engi.Message, **__):
    """
    Visualizza le ultime 10 transazioni del portafoglio di un membro.
    """

    transactions = _session.execute(
        ss.select(
            db.Transaction
        ).where(
            ss.or_(
                db.Transaction.minus == _target,
                db.Transaction.plus == _target,
            )
        ).order_by(
            db.Transaction.timestamp.desc()
        ).limit(
            10
        )
    ).scalars()

    msg = map(functools.partial(render, user=_target), transactions)

    await _msg.reply(text="\n\n".join(msg))


@rb.capture_errors
@engi.use_database(db.lazy_session_class)
@rb.use_ryglogin(allow_anonymous=False)
@rb.with_target()
@engi.TeleportingConversation
async def fiorygi_give(
        *,
        _user: db.User,
        _target: db.User,
        _msg: engi.Message,
        _session: db.SessionType,
        amount: int,
        reason: str,
        **__
):
    """
    Dai dei fiorygi a un altro membro.
    """

    if amount <= 0:
        await _msg.reply(text=f"⚠️ Puoi trasferire solo numeri interi positivi di fiorygi.")
        return

    if _user.fiorygi < amount:
        await _msg.reply(text=f"⚠️ Non hai sufficienti fiorygi per effettuare il trasferimento.")
        return

    if _user == _target:
        await _msg.reply(text=f"⚠️ Non puoi dare fiorygi a te stesso!")
        return

    trans = db.Transaction(
        minus=_user,
        plus=_target,
        amount=amount,
        reason=reason,
    )
    _session.add(trans)

    _user.fiorygi -= amount
    _target.fiorygi += amount

    _session.commit()

    await _msg.reply(text=f"💸 Hai trasferito \uE01Bƒ {amount}\uE00B a {_target}.")


@rb.capture_errors
@engi.use_database(db.lazy_session_class)
@rb.use_ryglogin(allow_anonymous=False)
@rb.with_target()
@engi.TeleportingConversation
async def fiorygi_magick(
        *,
        _user: db.User,
        _target: db.User,
        _msg: engi.Message,
        _session: db.SessionType,
        amount: int,
        reason: str,
        **__
):
    """
    Modifica il portafoglio di fiorygi di un membro.
    """

    if _user.sub != "auth0|5ed2debf7308300c1ea230c3":
        await _msg.reply(text=f"⚠️ Non sei autorizzato ad eseguire questo comando.")
        return

    if amount >= 0:
        trans = db.Transaction(
            minus=None,
            plus=_target,
            amount=amount,
            reason=reason,
        )
    else:
        trans = db.Transaction(
            minus=_target,
            plus=None,
            amount=amount,
            reason=reason,
        )

    _session.add(trans)

    _target.fiorygi += amount

    _session.commit()

    await _msg.reply(text=f"🏦 Hai modificato il portafoglio di {_target} di \uE01Bƒ {amount}\uE00B.")


@rb.capture_errors
@engi.use_database(db.lazy_session_class)
@rb.use_ryglogin(allow_anonymous=False)
@engi.TeleportingConversation
async def fiorygi_dig(
        *,
        _user: db.User,
        _session: db.SessionType,
        _msg: engi.Message,
        slug: str,
        **__
):
    """
    Cerca un tesoro con un dato nome.
    """

    treasure: t.Optional[db.Treasure] = _session.execute(
        ss.select(
            db.Treasure
        ).where(
            db.Treasure.slug == slug
        )
    ).scalar()

    if treasure is None:
        await _msg.reply(text=f"🏝 Non hai trovato nulla.")
        return

    if treasure.finder is not None:
        await _msg.reply(text=f"🏝 Quel tesoro è già stato trovato da {treasure.finder}.")
        return

    treasure.finder = _user
    treasure.find_time = arrow.now()

    trans = db.Transaction(
        minus=None,
        plus=_user,
        amount=treasure.value,
        reason=f"""🏖 Ritrovamento del tesoro `{treasure.slug}`""",
    )
    _session.add(trans)
    _user.fiorygi += treasure.value

    _session.commit()

    if treasure.message:
        await _msg.reply(text=f'🏖 Hai trovato un tesoro nascosto dal valore di \uE01Bƒ {treasure.value}\uE00B '
                              f'con il seguente messaggio:\n\n"{treasure.message}"')
    else:
        await _msg.reply(text=f"🏖 Hai trovato un tesoro nascosto dal valore di \uE01Bƒ {treasure.value}\uE00B!")


@rb.capture_errors
@engi.use_database(db.lazy_session_class)
@rb.use_ryglogin(allow_anonymous=False)
@engi.TeleportingConversation
async def fiorygi_bury(
        *,
        _user: db.User,
        _session: db.SessionType,
        _msg: engi.Message,
        slug: str,
        value: int,
        message: t.Optional[str],
        **__
):
    """
    Nascondi un tesoro con un dato nome.
    """

    treasure: t.Optional[db.Treasure] = _session.execute(
        ss.select(
            db.Treasure
        ).where(
            db.Treasure.slug == slug
        )
    ).scalar()

    if treasure is not None:
        if treasure.finder is not None:
            await _msg.reply(text=f"🏝 Un tesoro con quel nome è già stato trovato da {treasure.finder}.")
        else:
            await _msg.reply(text=f"🏖 Un tesoro con quel nome esiste già, "
                                  f"\uE01Bma non è ancora stato trovato da nessuno\uE00B!")
        return

    if value <= 0:
        await _msg.reply(text=f"⚠️ Il valore del tesoro deve essere intera positiva.")
        return

    if _user.fiorygi < value:
        await _msg.reply(text=f"⚠️ Non hai abbastanza fiorygi da nascondere.")
        return

    treasure = db.Treasure(
        slug=slug,
        creator=_user,
        creation_time=arrow.now(),
        value=value,
        message=message,
    )
    _session.add(treasure)

    trans = db.Transaction(
        minus=_user,
        plus=None,
        amount=treasure.value,
        reason=f"""🏖 Occultamento di un tesoro""",
    )
    _session.add(trans)
    _user.fiorygi -= value

    _session.commit()

    if treasure.message:
        await _msg.reply(text=f'🏖 Hai nascosto un tesoro dal valore di \uE01Bƒ {treasure.value}\uE00B '
                              f'con il seguente messaggio:\n"{treasure.message}"')
    else:
        await _msg.reply(text=f'🏖 Hai nascosto un tesoro dal valore di \uE01Bƒ {treasure.value}\uE00B.')


__all__ = (
    "fiorygi_balance_self",
    "fiorygi_balance_other",
    "fiorygi_give",
    "fiorygi_magick",
    "fiorygi_transactions_self",
    "fiorygi_transactions_other",
    "fiorygi_dig",
    "fiorygi_bury",
)



import royalnet.engineer as engi
import royalspells
import royalpack.bolts as rb


@rb.capture_errors
@engi.TeleportingConversation
async def spell(*, _msg: engi.Message, spellname: str, **__):
    """
    Genera una spell casuale!
    """
    s = royalspells.Spell(spellname)

    rows: list[str] = [f"✨ \uE01B{s.name}\uE00B"]

    if s.damage_component:
        dmg: royalspells.DamageComponent = s.damage_component
        constant_str: str = f"{dmg.constant:+d}" if dmg.constant != 0 else ""
        rows.append(f"Danni: {dmg.dice_number}d{dmg.dice_type}{constant_str}"
                    f" {', '.join(dmg.damage_types)}")
        rows.append(f"Precisione: {dmg.miss_chance}%")
        if dmg.repeat > 1:
            rows.append(f"Multiattacco: ×{dmg.repeat}")
        rows.append("")

    if s.healing_component:
        heal: royalspells.HealingComponent = s.healing_component
        constant_str: str = f"{heal.constant:+d}" if heal.constant != 0 else ""
        rows.append(f"Cura: {heal.dice_number}d{heal.dice_type}{constant_str} HP")
        rows.append("")

    if s.stats_component:
        stats: royalspells.StatsComponent = s.stats_component
        rows.append("Il caster riceve: ")
        for stat_name in stats.stat_changes:
            rows.append(f"{stat_name}{stats.stat_changes[stat_name]}")
        rows.append("")

    if s.status_effect_component:
        se: royalspells.StatusEffectComponent = s.status_effect_component
        rows.append("Infligge al bersaglio: ")
        rows.append(f"{se.effect} ({se.chance}%)")
        rows.append("")

    await _msg.reply(text="\n".join(rows))


__all__ = ("spell",)



import sqlalchemy.orm
import royalnet.lazy

from ..config import *


lazy_engine = royalnet.lazy.Lazy(lambda c: sqlalchemy.create_engine(c["database.uri"]), c=lazy_config)
"""
The uninitialized sqlalchemy engine.
"""

lazy_session_class = royalnet.lazy.Lazy(lambda e: sqlalchemy.orm.sessionmaker(bind=e), e=lazy_engine)
"""
The uninitialized sqlalchemy session class.
"""

SessionType = sqlalchemy.orm.Session

__all__ = (
    "lazy_engine",
    "lazy_session_class",
    "SessionType",
)



"""

"""

from __future__ import annotations

import functools
import logging

import royalnet.engineer as engi
import royalnet_discordpy
import royalnet_telethon
import sqlalchemy.orm as so
import sqlalchemy.sql as ss

import royalpack.database as db

log = logging.getLogger(__name__)


def use_ryglogin(allow_anonymous=False):
    """
    Decorator factory which allows a :class:`~royalnet.engineer.conversation.Conversation` to find out what RYGaccount
    user called a certain function.

    It requires the :func:`~royalnet.engineer.bolts.use_database` decorator.

    :param allow_anonymous: If users who are not logged in should be allowed to use the command, or should be displayed
                            an error instead.
    :return: The decorator to use to decorate the function.
    """

    def decorator(f):
        @functools.wraps(f)
        async def decorated(_session: so.Session, _imp: engi.PDAImplementation, _msg: engi.Message, **f_kwargs):

            if isinstance(_imp, royalnet_telethon.TelethonPDAImplementation):
                _sender = await _msg.sender
                supported = True
                account = _session.execute(
                    ss.select(db.TelegramAccount).where(db.TelegramAccount.id == _sender._user.id)
                ).scalar()

            elif isinstance(_imp, royalnet_discordpy.DiscordpyPDAImplementation):
                _sender = await _msg.sender
                supported = True
                account = _session.execute(
                    ss.select(db.DiscordAccount).where(db.DiscordAccount.id == _sender._user.id)
                ).scalar()

            else:
                supported = False
                account = None

            if account:
                user = account.user
            else:
                if allow_anonymous:
                    user = None
                else:
                    if supported:
                        await _msg.reply(text=f"⚠️ Non sei loggato al tuo RYGaccount! "
                                              f"Usa il comando login per effettuare la connessione.")
                    else:
                        await _msg.reply(text=f"⚠️ Il login non è disponibile su questa implementazione di Royalnet, "
                                              f"e questo comando non consente l'esecuzione anonima.")
                    return

            return await f(_session=_session, _imp=_imp, _msg=_msg, **f_kwargs, _account=account, _user=user)

        return decorated

    return decorator


__all__ = (
    "use_ryglogin",
)



import royalnet.engineer as engi
import royalpack.bolts as rb


@rb.capture_errors
@engi.TeleportingConversation
async def pmots(*, _msg: engi.Message, **__):
    """
    Riprenditi da uno stomp colossale!

    https://t.me/c/1153723135/181784
    """
    await _msg.reply(text="👣 pmots pmots")


__all__ = ("pmots",)



"""Make find_time nullable

Revision ID: 113c2444dad4
Revises: e6b0d97063a1
Create Date: 2021-04-25 23:17:50.846179

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql

# revision identifiers, used by Alembic.
revision = '113c2444dad4'
down_revision = 'e6b0d97063a1'
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.alter_column('fiorygi_treasures', 'find_time',
               existing_type=postgresql.TIMESTAMP(),
               nullable=True)
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.alter_column('fiorygi_treasures', 'find_time',
               existing_type=postgresql.TIMESTAMP(),
               nullable=False)
    # ### end Alembic commands ###



import discord
import pkg_resources
import royalnet.engineer as engi
import royalnet.scrolls as sc
import royalnet_telethon as rt
import royalnet_discordpy as rd
import pathlib
import re
import coloredlogs
import sentry_sdk
import sentry_sdk.integrations.atexit
import sentry_sdk.integrations.dedupe
import sentry_sdk.integrations.modules
import sentry_sdk.integrations.threading
import logging

from . import commands
from .database import engine, base

coloredlogs.install(level="DEBUG" if __debug__ else "INFO", isatty=True)
config = sc.Scroll.from_file(namespace="ROYALPACK", file_path=pathlib.Path("royalpack.cfg.toml"))

if dsn := config.get("sentry.dsn", None):
    logging.info("Enabling Sentry...")
    sentry_sdk.init(
        dsn=dsn,
        debug=__debug__,
        release=pkg_resources.get_distribution("royalpack").version,
        environment="Development" if __debug__ else "Production",
        default_integrations=False,
        integrations=[
            sentry_sdk.integrations.atexit.AtexitIntegration(),
            sentry_sdk.integrations.dedupe.DedupeIntegration(),
            sentry_sdk.integrations.modules.ModulesIntegration(),
            sentry_sdk.integrations.threading.ThreadingIntegration(),
        ],
        traces_sample_rate=1.0
    )

engine_ = engine.lazy_engine.evaluate()
base.Base.metadata.create_all(engine_)

pda = engi.PDA(implementations=[
    rt.TelethonPDAImplementation(
        name="1",
        tg_api_id=config["telegram.api.id"],
        tg_api_hash=config["telegram.api.hash"],
        bot_username=config["telegram.bot.username"],
        bot_token=config["telegram.bot.token"],
    ),
    rd.DiscordpyPDAImplementation(
        name="2",
        bot_token=config["discord.bot.token"],
        intents=discord.Intents.all(),
    ),
])


def register_telegram(router, conv, names, syntax=None):
    name_regex = rf"(?:{'|'.join(names)})"
    bot_regex = rf"(?:@{config['telegram.bot.username']})?"
    if syntax:
        syntax_regex = rf"\s+{syntax}"
    else:
        syntax_regex = ""
    regex = rf"^/{name_regex}{bot_regex}{syntax_regex}$"
    router.register_conversation(conv, names, [re.compile(regex)])


def register_discord(router, conv, names, syntax=None):
    name_regex = rf"(?:{'|'.join(names)})"
    if syntax:
        syntax_regex = rf"\s+{syntax}"
    else:
        syntax_regex = ""
    prefix_regex = rf"{config['discord.bot.prefix']}"
    regex = rf"^{prefix_regex}{name_regex}{syntax_regex}$"
    router.register_conversation(conv, names, [re.compile(regex)])


tg_router = engi.Router()

register_telegram(tg_router, commands.ahnonlosoio, ["ahnonlosoio"])
register_telegram(tg_router, commands.answer, ["answer"], r".+")
register_telegram(tg_router, commands.cat, ["cat", "catto", "gatto", "nyaa", "nya"])
register_telegram(tg_router, commands.ciaoruozi, ["ciaoruozi"])
register_telegram(tg_router, commands.color, ["color"])
register_telegram(tg_router, commands.ping, ["ping"])
register_telegram(tg_router, commands.ship, ["ship"], r"(?P<first>[A-Za-z]+)[\s+&]+(?P<second>[A-Za-z]+)")
register_telegram(tg_router, commands.emojify, ["emojify"], r"(?P<message>.+)")
register_telegram(tg_router, commands.dog_any, ["dog", "doggo", "cane", "woof", "bau"])
register_telegram(tg_router, commands.dog_breedlist, ["dog", "doggo", "cane", "woof", "bau"], r"(?:list|help|aiuto)")
register_telegram(tg_router, commands.dog_breed, ["dog", "doggo", "cane", "woof", "bau"], r"(?P<breed>[A-Za-z/]+)")
register_telegram(tg_router, commands.fortune, ["fortune"])
register_telegram(tg_router, commands.pmots, ["pmots"])
register_telegram(tg_router, commands.spell, ["spell", "cast"], r"(?P<spellname>.+)")
register_telegram(tg_router, commands.smecds, ["smecds"])
register_telegram(tg_router, commands.man, ["man", "help"], r"(?P<commandname>[A-Za-z]+)")
register_telegram(tg_router, commands.login, ["login"])
register_telegram(tg_router, commands.whoami, ["whoami"])
register_telegram(tg_router, commands.fiorygi_balance_self, ["balance"])
register_telegram(tg_router, commands.fiorygi_balance_other, ["balance"], r"(?P<target>\S+)")
register_telegram(tg_router, commands.fiorygi_give, ["give"], r"(?P<target>\S+)\s+(?P<amount>[0-9]+)\s+(?P<reason>.+)")
register_telegram(tg_router, commands.fiorygi_magick, ["magick"], r"(?P<target>\S+)\s+(?P<amount>[0-9]+)\s+(?P<reason>.+)")
register_telegram(tg_router, commands.fiorygi_transactions_self, ["transactions"])
register_telegram(tg_router, commands.fiorygi_transactions_other, ["transactions"], r"(?P<target>\S+)")
register_telegram(tg_router, commands.fiorygi_dig, ["dig"], r"(?P<slug>[a-z0-9-]+)")
register_telegram(tg_router, commands.fiorygi_bury, ["bury"], r"(?P<slug>[a-z0-9-]+)\s+(?P<value>[0-9]+)(?:\s+(?P<message>.+))?")
register_telegram(tg_router, commands.version, ["version"])
register_telegram(tg_router, commands.cv, ["cv", "civù"])

ds_router = engi.Router()

register_discord(ds_router, commands.ahnonlosoio, ["ahnonlosoio"])
register_discord(ds_router, commands.answer, ["answer"], r".+")
register_discord(ds_router, commands.cat, ["cat", "catto", "gatto", "nyaa", "nya"])
register_discord(ds_router, commands.ciaoruozi, ["ciaoruozi"])
register_discord(ds_router, commands.color, ["color"])
register_discord(ds_router, commands.ping, ["ping"])
register_discord(ds_router, commands.ship, ["ship"], r"(?P<first>[A-Za-z]+)[\s+&]+(?P<second>[A-Za-z]+)")
register_discord(ds_router, commands.emojify, ["emojify"], r"(?P<message>.+)")
register_discord(ds_router, commands.dog_any, ["dog", "doggo", "cane", "woof", "bau"])
register_discord(ds_router, commands.dog_breedlist, ["dog", "doggo", "cane", "woof", "bau"], r"(?:list|help|aiuto)")
register_discord(ds_router, commands.dog_breed, ["dog", "doggo", "cane", "woof", "bau"], r"(?P<breed>[A-Za-z/]+)")
register_discord(ds_router, commands.fortune, ["fortune"])
register_discord(ds_router, commands.pmots, ["pmots"])
register_discord(ds_router, commands.spell, ["spell", "cast"], r"(?P<spellname>.+)")
register_discord(ds_router, commands.smecds, ["smecds"])
register_discord(ds_router, commands.man, ["man", "help"], r"(?P<commandname>[A-Za-z]+)")
register_discord(ds_router, commands.login, ["login"])
register_discord(ds_router, commands.whoami, ["whoami"])
register_discord(ds_router, commands.fiorygi_balance_self, ["balance"])
register_discord(ds_router, commands.fiorygi_balance_other, ["balance"], r"(?P<target>\S+)")
register_discord(ds_router, commands.fiorygi_give, ["give"], r"(?P<target>\S+)\s+(?P<amount>[0-9]+)\s+(?P<reason>.+)")
register_discord(ds_router, commands.fiorygi_magick, ["magick"], r"(?P<target>\S+)\s+(?P<amount>[0-9]+)\s+(?P<reason>.+)")
register_discord(ds_router, commands.fiorygi_transactions_self, ["transactions"])
register_discord(ds_router, commands.fiorygi_transactions_other, ["transactions"], r"(?P<target>\S+)")
register_discord(ds_router, commands.fiorygi_dig, ["dig"], r"(?P<slug>[a-z0-9-]+)")
register_discord(ds_router, commands.fiorygi_bury, ["bury"], r"(?P<slug>[a-z0-9-]+)\s+(?P<value>[0-9]+)(?:\s+(?P<message>.+))?")
register_discord(ds_router, commands.version, ["version"])

pda.implementations["telethon.1"].register_conversation(tg_router)
pda.implementations["discordpy.2"].register_conversation(ds_router)


pda.run()



"""

"""

from __future__ import annotations

import functools
import logging
import sentry_sdk

log = logging.getLogger(__name__)


def capture_errors(f):
    @functools.wraps(f)
    async def decorated(**f_kwargs):
        try:
            return await f(**f_kwargs)
        except Exception as e:
            log.error(f"Captured error: {e!r}")
            if sentry_sdk.Hub.current is not None:
                sentry_sdk.capture_exception(error=e)

    return decorated


__all__ = (
    "capture_errors",
)



import logging.config
import sqlalchemy
import sqlalchemy.pool
import alembic

from royalpack.database.base import Base
from royalpack.database.engine import lazy_engine
from royalpack.config import lazy_config


# Set up logging
logging.config.fileConfig(alembic.context.config.config_file_name)


# Get the metadata
target_metadata = Base.metadata


def run_migrations_offline():
    """Run migrations in 'offline' mode.

    This configures the context with just a URL
    and not an Engine, though an Engine is acceptable
    here as well.  By skipping the Engine creation
    we don't even need a DBAPI to be available.

    Calls to context.execute() here emit the given string to the
    script output.

    """
    royalpack_config = lazy_config.evaluate()

    alembic.context.configure(
        url=royalpack_config["database.uri"],
        target_metadata=target_metadata,
        literal_binds=True,
        dialect_opts={"paramstyle": "named"},
    )

    with alembic.context.begin_transaction():
        alembic.context.run_migrations()


def run_migrations_online():
    """Run migrations in 'online' mode.

    In this scenario we need to create an Engine
    and associate a connection with the context.

    """
    engine = lazy_engine.evaluate()

    with engine.connect() as connection:
        alembic.context.configure(
            connection=connection, target_metadata=target_metadata
        )

        with alembic.context.begin_transaction():
            alembic.context.run_migrations()


if alembic.context.is_offline_mode():
    run_migrations_offline()
else:
    run_migrations_online()



import royalnet.engineer as engi
import royalnet_telethon
import royalnet_telethon.bullet.contents
import royalpack.bolts as rb


@rb.capture_errors
@engi.TeleportingConversation
async def ciaoruozi(*, _msg: engi.Message, _imp, **__):
    """
    Saluta Ruozi, una creatura leggendaria che potrebbe esistere o non esistere in Royal Games.
    """

    if isinstance(_imp, royalnet_telethon.TelethonPDAImplementation):
        sender: royalnet_telethon.bullet.contents.TelegramUser = await _msg.sender
        # noinspection PyProtectedMember
        if sender._user.id == 112437036:
            await _msg.reply(text="👋 Ciao \uE01Bme\uE00B!")

    await _msg.reply(text="👋 Ciao Ruozi!")


__all__ = ("ciaoruozi",)



import royalnet.engineer as engi
import random
import datetime
import royalpack.bolts as rb


# Tutte le fortunes qui devono essere positive :)
# O almeno neutrali...
_fortunes = [
    "😄 Oggi sarà una fantastica giornata!",
    "😌 Oggi sarà una giornata molto chill e rilassante.",
    "💰 Oggi sui tuoi alberi cresceranno più Stelline!",
    "🍎 Oggi un unicorno ti lascerà la sua Blessed Apple!",
    "📈 Oggi il tuo team in ranked sarà più amichevole e competente del solito!",
    "🏝 Oggi potrai raggiungere l'Isola Miraggio!",
    "🐱 Oggi vedrai più gatti del solito su Internet!",
    "🐶 Oggi vedrai più cani del solito su Internet!",
    "🐦 Oggi vedrai più uccelli del solito su Internet!",
    "🐌 Oggi incontrerai una chiocciola sperduta!",
    "🎁 Oggi i dispenser di regali in centro funzioneranno senza problemi!",
    "🥕 Oggi il tuo raccolto avrà qualità Iridium Star!",
    "🔴 Oggi troverai più oggetti di rarità rossa del solito!",
    "✨ Oggi farai molti più multicast!",
    "♦️ Oggi troverai una Leggendaria Dorata!",
    "⭐️ Oggi la stella della RYG ti sembrerà un pochino più dritta!",
    "⭐️ Oggi la stella della RYG ti sembrerà anche più storta del solito!",
    "💎 Oggi i tuoi avversari non riusciranno a deflettere i tuoi Emerald Splash!",
    "⬅️ Oggi le tue supercazzole prematureranno un po' più a sinistra!",
    "➡️ Oggi le tue supercazzole prematureranno un po' più a destra!",
    "🌅 Oggi sarà il giorno dopo ieri e il giorno prima di domani!",
    "🤖 Oggi il Royal Bot ti dirà qualcosa di molto utile!",
    "🏠 Oggi qualcuno si autoinviterà a casa tua!",
    "📵 Oggi passerai una bella giornata tranquilla senza che nessuno ti chiami!",
    "🕸 Oggi cadrai trappola di una ragnatela! \uE011O ti arriverà in faccia.\uE001",
    "🔮 Oggi chiederai a @royalgamesbot di dirti la tua /fortune!",
    "👽 Oggi incontrerai gli UFI!!!1!!uno!",
    "🦾 Oggi uno scienziato pazzo ti proporrà di sostituire il tuo braccio con un braccio-razzo meccanico!",
    "🕵️ Oggi una spia in incognito ti chiederà se hai mai visto the Emoji Movie!",
    "🍕 Oggi mangerai una margherita doppio pomodoro!",
    "🍰 Oggi mangerai una torta al gusto di torta!",
    "🥇 Oggi vincerai qualcosa!",
    "🏴‍☠️ Oggi salperai i sette mari con la tua ciurma pirata!",
    "🕒 Oggi sarà ieri, e domani sarà oggi!",
    "🔙 Oggi tornerai indietro nel tempo!",
    "🚨 Oggi suonerà l'allarme della Velvet Room!",
    "🏳️‍🌈 Oggi scoprirai l'esistenza di almeno un gender che non conoscevi!",
    "🥴 Oggi ti dimenticherai come ci si siede!",
    "👀 Oggi scoprirai di avere degli occhi!",
    "🏹 Oggi ti verrà voglia di installare Arch Linux, ma cambierai idea molto in fretta!",
    "🩲 Oggi annuncerai alla cv di essere in mutande!",
    "👟 Oggi tua madre ti regalerà delle scarpe da corsa!",
    "✨ Oggi troverai un Pokémon shiny!",
    "👏 Oggi sarai felice, lo saprai e batterai le mani!",
    "🦴 Oggi scoprirai di avere uno scheletro wholesome all'interno di te!",
    "💳 Oggi riuscirai a fornire i tre numerini della tua carta di credito a John Wick!",
    "🤔 Oggi smetterai finalmente di essere sus, in quanto sarai confermato dal villaggio!",
]


@rb.capture_errors
@engi.TeleportingConversation
async def fortune(*, _msg: engi.Message, **__):
    """
    Come sarà la giornata di oggi?
    """

    author = await _msg.sender

    r = random.Random(x=hash(author) + hash(datetime.date.today()))
    message = r.sample(_fortunes, 1)[0]

    await _msg.reply(text=message)


__all__ = ("fortune",)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['royalpack',
 'royalpack.bolts',
 'royalpack.commands',
 'royalpack.database',
 'royalpack.database.alembic',
 'royalpack.database.alembic.versions']

package_data = \
{'': ['*']}

install_requires = \
['SQLAlchemy-Utils>=0.37.0,<0.38.0',
 'aiohttp>=3.7.4,<4.0.0',
 'alembic>=1.5.8,<2.0.0',
 'arrow>=1.0.3,<2.0.0',
 'async-timeout>=3.0.1,<4.0.0',
 'coloredlogs>=15.0,<16.0',
 'colour>=0.1.5,<0.2.0',
 'psycopg2>=2.8.6,<3.0.0',
 'royalnet-discordpy>=0.5.8,<0.6.0',
 'royalnet>=6.5.6,<6.6.0',
 'royalnet_console>=2.0.4,<3.0.0',
 'royalnet_telethon>=2.0.0,<3.0.0',
 'royalspells>=3.2,<4.0',
 'sentry-sdk>=1.0.0,<2.0.0',
 'steam>=1.2.0,<2.0.0']

setup_kwargs = {
    'name': 'royalpack',
    'version': '6.0.5',
    'description': 'Royalnet Commands for the RYG community',
    'long_description': '# Royalpack\n\n[Royalnet](https://github.com/Steffo99/royalnet-6) Commands for the RYG community\n',
    'author': 'Stefano Pigozzi',
    'author_email': 'me@steffo.eu',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/RYGhub/royalpack',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



from .ahnonlosoio import *
from .answer import *
from .cat import *
from .ciaoruozi import *
from .color import *
from .ping import *
from .ship import *
from .emojify import *
from .dog import *
from .fortune import *
from .pmots import *
from .spell import *
from .smecds import *
from .man import *
from .login import *
from .whoami import *
from .fiorygi import *
from .version import *
from .cv import *



import royalnet.royaltyping as t
import royalnet.engineer as engi
import royalnet_discordpy as rd
import royalpack.bolts as rb
import discord
import discord.channel
import itertools


@rb.capture_errors
@engi.TeleportingConversation
async def cv(*, _msg: engi.Message, _pda: engi.PDA, **__):
    """
    Visualizza chi è connesso in chat vocale!
    """
    imp = _pda.implementations["discordpy.2"]
    assert isinstance(imp, rd.DiscordpyPDAImplementation)
    await imp.client.wait_until_ready()

    guild: discord.Guild = imp.client.get_guild(176353500710699008)

    message = []

    members_by_vc = get_members_by_voice_channel(guild=guild)

    for channel in get_interesting_channels(guild=guild):
        members_in_channel = members_by_vc.get(channel, [])
        if len(members_in_channel) == 0:
            continue

        message.append(build_channel_string(channel=channel))
        message.append("")

        for member in members_in_channel:
            message.append(build_member_string(member=member))

        message.append("")
        message.append("")

    if len(message) == 0:
        message.append("☁️ \uE011Non c'è nessuno in chat vocale.\uE001")

    await _msg.reply(text="\n".join(message))


def get_interesting_channels(guild: discord.Guild) -> list[discord.channel.VocalGuildChannel]:
    # noinspection PyTypeChecker
    return list(filter(lambda c: isinstance(c, discord.channel.VocalGuildChannel), guild.channels))


def get_voice_channel(member: t.Union[discord.Member, discord.User]) -> t.Optional[discord.channel.VocalGuildChannel]:
    if not member.voice:
        return None
    return member.voice.channel


def key_is_position(member: t.Union[discord.Member, discord.User]) -> int:
    if not member.voice:
        return 0
    return -member.voice.channel.position


def get_members_by_voice_channel(guild: discord.Guild):
    members = sorted(guild.members, key=key_is_position)
    groups = itertools.groupby(members, key=get_voice_channel)
    result = {}
    for group in groups:
        result[group[0]] = list(group[1])
    return result


def build_channel_string(channel: discord.channel.VocalGuildChannel) -> str:
    if isinstance(channel, discord.VoiceChannel):
        emoji = "🎙"
    elif isinstance(channel, discord.StageChannel):
        emoji = "📡"
    else:
        emoji = "❔"

    name = channel.name

    return f"\uE01B{emoji} {name}\uE00B"


def build_member_string(member: t.Union[discord.Member, discord.User]) -> str:
    _status: t.Union[discord.Status, str] = member.status
    if _status == discord.Status.online:
        status = "🔵"
    elif _status == discord.Status.dnd:
        status = "🔴"
    elif _status == discord.Status.idle:
        status = "⚫️"
    elif _status == discord.Status.offline:
        status = "⚪️"
    else:
        status = "❔"

    _voice: discord.VoiceState = member.voice
    if not _voice:
        voice = ""
    elif _voice.requested_to_speak_at:
        voice = "👋"
    elif _voice.self_deaf:
        voice = "🔇"
    elif _voice.self_mute:
        voice = "🔈"
    elif _voice.deaf:
        voice = "⛔️"
    elif _voice.mute:
        voice = "⛔️"
    elif _voice.self_stream:
        voice = "🖥"
    elif _voice.self_video:
        voice = "📹"
    else:
        voice = "🔊"

    if member.nick:
        name = member.nick
    else:
        name = member.name

    return "\n".join([
        f"{status}{voice} \uE012{name}\uE002",
        *map(
            lambda activity: build_activity_string(activity=activity),
            filter(
                lambda activity: activity.name, member.activities
            )
        )
    ])


def build_activity_string(activity: discord.Activity) -> str:
    if activity.type == discord.ActivityType.custom:
        text = f"\uE011{activity.name}\uE001"
    elif activity.type == discord.ActivityType.streaming:
        text = f"{activity.url}"
        breakpoint()
    else:
        text = f"{activity.name}"

    try:
        if activity.type == discord.ActivityType.listening:
            # noinspection PyUnresolvedReferences
            extra = f"({', '.join(activity.artists)} - {activity.album})"
        elif activity.state and activity.details:
            extra = f"({activity.state} | {activity.details})"
        elif activity.state:
            extra = f"({activity.state})"
        elif activity.details:
            extra = f"({activity.details})"
        else:
            extra = ""
    except AttributeError:
        extra = ""

    return f"- {text} {extra}"


__all__ = ("cv",)



import pathlib
import royalnet.scrolls as s
import royalnet.lazy as l


lazy_config = l.Lazy(lambda: s.Scroll.from_file("ROYALPACK", pathlib.Path("royalpack.cfg.toml")))


__all__ = (
    "lazy_config",
)
